--------------------------------------------------------------------------------
-- Copyright  2021								|
-- All Rights Reserved.								|
-- 										|
-- Permission to use, copy, modify, and distribute this software in any way	|
-- is not granted without expressed written agreement. This copyright applies	|
-- to the all code, data and artwork related to this project and all associated	|
-- projects published by the author.						|
--										|
-- More information on copyright is available from sources on-line.		|
-- Use an actual factual one!							|
--------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------
-- 			Meter configuration and assignment get run for each new character.			--
-- 			It's up to the meter author to figure out what needs to be				--
-- 			run once and what needs to be run each time.						--
------------------------------------------------------------------------------------------------------------------
local _, NS = ...

local AddonName = "Meters"

local len, sub, upper = string.len, string.sub, string.upper

local PlayerName = UnitName("player")
local RealmName = GetRealmName()

local MeterDefaults

local function ConfigureMeters(reset)
	MeterDefaults = MeterDefaults or NS:GetOption("GetDefaultAccessoryList")("Meter")
--	GetMeters()
	local Settings = NS:GetSettings("Addons").Meters --NS:GetSettings("Meters")
	for k, v in pairs(MeterDefaults) do
		if not Settings[k] or (Settings[k] and reset) then -- Settings[k] == "none" for "I don't want no stinking meter of this type!!!"
			local Addon = NS:GetOption("GetDefaultAccessory")("Meter", k)
--			local Addon
--			if MeterList[k] then -- Check the installed meter addons for the FM default of each type
--				if MeterList[k] and MeterList.sortlist[MeterDefaults[k]] then -- FauxMazzle default exists
--					Addon = MeterDefaults[k] -- Default found
--				else
--					Addon = MeterList[k][1].addon -- just take the first in the list
----					MeterDefaults[k] = Addon -- Update the global default meter??
--				end
--			end

			if Settings.inactive or (Settings[k] and Settings[k].inactive) then -- All Meters or this meter set to inactive
				Addon = nil
			end
			if Addon then
				local AddonSettings = NS:GetOption("GetAccessorySettings")(Addon, "Meter", k)
				if AddonSettings then
					if Settings[k] and Settings[k].addon == Addon and AddonSettings.Frame then
						local f = _G[AddonSettings.Frame]
						if f then
							f:SetParent(UIParent)
							f:ClearAllPoints()
							f:SetPoint("LEFT", -5, 0)
							f:SetSize(2, 2)
						end
					end
					if AddonSettings.Run then
						AddonSettings.Run(PlayerName, RealmName)
					end
					if not Settings[k] then
						Settings[k] = {}
					end
					Settings[k].addon = Addon
				end
			else
				if Settings[k] then
					Settings[k].inactive = true
				end
			end
		end
	end
----- Moved Here
	local Found
	for k, v in pairs(MeterDefaults) do
--		if MeterDefaults[k] and not Settings[k] then
--			Settings[k] = { addon=MeterDefaults[k] }
--		end
		if Settings[k] and not Settings[k].inactive then
			Found = Found or NS:GetOption("LoadAccessory")(Settings[k].addon, "Meter", k)
		end
	end
	if Found then -- We found at least one meter
		NS:GetOption("PositionMeters")()
	end
-- End Move
end
--NS:AddConfig(3, AddonName, "Meters", ConfigureMeters, AssignMeters)
NS:AddConfig(3, AddonName, "Meters", ConfigureMeters, ConfigureMeters) -- Added first ConfigureMeters to enable Meters in Reset list.

local function MeterOptions(parent, addonlistentry, offset)
	if addonlistentry.frame then
		addonlistentry.frame:Show()
		return addonlistentry.frame
	end
	local L = NS.TEXTS
	local Container = CreateFrame("Frame", "MeterOptions", parent)
	NS.AddBackground(Container)
	Container:SetSize(140, parent:GetHeight())
	
	local DamageList = NS:GetOption("GetAccessoryList")("meter", "damage")
	local ThreatList = NS:GetOption("GetAccessoryList")("meter", "threat")
	local DamageMeters = {}
	local ThreatMeters = {}
	if DamageList then
		local DefaultDamage = NS:GetOption("GetDefaultAccessory")("meter", "damage")
		for k, v in pairs(DamageList) do
			if k ~= DefaultDamage then
				tinsert(DamageMeters, { title=v.title..v.author, value=k, })
			end
		end
		if DefaultDamage then
			local DefaultDamageOptions = NS:GetOption("GetAccessoryInfo")(DefaultDamage, "meter", "damage")
			tinsert(DamageMeters, 1, { title=DefaultDamageOptions.title..DefaultDamageOptions.author, value=DefaultDamage, })
		end
	end
	if ThreatList then
		local DefaultThreat = NS:GetOption("GetDefaultAccessory")("meter", "threat")
		local DefaultThreatOptions = NS:GetOption("GetAccessoryInfo")(DefaultThreat, "meter", "threat")
		for k, v in pairs(ThreatList) do
			if k ~= DefaultThreat then
				tinsert(ThreatMeters, { title=v.title..v.author, value=k, })
			end
		end
		if DefaultThreat then
			local DefaultThreatOptions = NS:GetOption("GetAccessoryInfo")(DefaultThreat, "meter", "threat")
			tinsert(ThreatMeters, 1, { title=DefaultThreatOptions.title..DefaultThreatOptions.author, value=DefaultThreat, })
		end
	end
	local Relative = Container
	local YOffset = -20
	local function CreateMeterList(name, parent, addon, key, table, label)
		local Settings = NS:GetSettings("Addons")
		parent[name.."Label"] = NS:CreateLabel(parent, name.."Label", label)
		parent[name.."Label"]:SetPoint("TOP", Relative, -10, YOffset)
		local f = NS:CreateDropList(name, parent, Settings[addon][key], "addon", table, "title", "value")
--		f:AddPadding(10)
--		f:SetLabel(label, "TOP", 0, 3)
		f:SetSize(130, 23)
		f:SetPoint("TOPLEFT", 0, YOffset + -13)
		f.InitValue = f.Data[f.Index]
		f.PostUpdate = function(self)
			self.Button.Icon:SetAlpha(0.5)
			self.Button.Icon:SetVertexColor(1, 1, 1)
			if self.Data[f.Index] ~= self.InitValue then
				FauxMazzleOptions:SetReloadHighlight(true)
				local Info = NS:GetOption("LoadAccessory")(self.Data[f.Index], "Meter", key)
				if Info.loaded then
					if Info.settings.Run then
						Info.settings.Run(PlayerName, RealmName)
					end
					self.Button.UpdateProfiles = Info.settings.UpdateProfiles
				end
			else
				FauxMazzleOptions:SetReloadHighlight()
			end
		end
		local b = NS:CreateUpdateAllButton(name.."Update", parent, addon, key, "addon")
		f.Button = b
		b:SetSize(80, 20)
		b:SetPoint("TOP", f, "BOTTOM", 0, -15)
		f:AddDropHider(b)
		return f, b
	end
	if DamageList then
		Container.DamageMeters = CreateMeterList("DamageMeters", Container, AddonName, "damage", DamageMeters, L.Damage)
		if #ThreatMeters > 0 then
			YOffset = -115
			Container.ThreatMeters = CreateMeterList("ThreatMeters", Container, AddonName, "threat", ThreatMeters, L.Threat)
		end
	end
	
	return Container
end
NS:RegisterAddonOptions(AddonName, MeterOptions)